<?php

namespace Buddycore;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Buddycore plugin.
 *
 * The main plugin handler class is responsible for initializing Buddycore. The
 * class registers and all the components required to run the plugin.
 *
 * @since 1.0.0
 */
class Plugin {

	public $components = [];

	/**
	 * Instance.
	 *
	 * Holds the plugin instance.
	 *
	 * @since 1.0.0
	 * @access public
	 * @static
	 *
	 * @var Plugin
	 */
	public static $instance = null;

	/**
	 * Clone.
	 *
	 * Disable class cloning and throw an error on object clone.
	 *
	 * The whole idea of the singleton design pattern is that there is a single
	 * object. Therefore, we don't want the object to be cloned.
	 *
	 * @access public
	 * @since 1.0.0
	 */
	public function __clone() {
		// Cloning instances of the class is forbidden.
		_doing_it_wrong( __FUNCTION__, esc_html__( 'Something went wrong.', 'buddycore' ), '1.0.0' );
	}

	/**
	 * Wakeup.
	 *
	 * Disable unserializing of the class.
	 *
	 * @access public
	 * @since 1.0.0
	 */
	public function __wakeup() {
		// Unserializing instances of the class is forbidden.
		_doing_it_wrong( __FUNCTION__, esc_html__( 'Something went wrong.', 'buddycore' ), '1.0.0' );
	}

	/**
	 * Instance.
	 *
	 * Ensures only one instance of the plugin class is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 * @access public
	 * @static
	 *
	 * @return Plugin An instance of the class.
	 */
	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();

			/**
			 * Buddycore loaded.
			 *
			 * Fires when Buddycore was fully loaded and instantiated.
			 *
			 * @since 1.0.0
			 */
			do_action( 'buddycore/loaded' );
		}

		return self::$instance;
	}

	/**
	 * Plugin constructor.
	 *
	 * Initializing Buddycore plugin.
	 *
	 * @since 1.0.0
	 * @access private
	 */
	private function __construct() {
		$this->register_autoloader();

		add_action( 'plugins_loaded', [ $this, 'init_components' ] );

		add_action( 'after_setup_theme', [ $this, 'after_setup_theme' ] );

		add_action( 'widgets_init', [ $this, 'register_widgets' ] );

		add_action( 'init', [ $this, 'init' ] );

	}


	/**
	 * Init.
	 *
	 * Initialize Buddycore Plugin. Register Buddycore support for all the
	 * supported post types and initialize Buddycore components.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function init() {

		/**
		 * Buddycore init.
		 *
		 * Fires after Buddycore init
		 *
		 * @since 1.0.0
		 */
		do_action( 'buddycore/init' );
	}

	public function after_setup_theme() {

		//Theme moved functions
		require_once BA_CORE_PATH . 'core/after-theme.php';

		// Resize on the fly
		require_once( BA_CORE_PATH . '/lib/aq_resizer.php' );
	}

	public function register_widgets() {

		register_widget( "\Buddycore\Widgets\Recent_Posts" );
		register_widget( "\Buddycore\Widgets\Social_Share" );
	}

	/**
	 * Init components.
	 *
	 * Initialize Buddycore components. Register actions.
	 * initialize admin components.
	 *
	 * @since 1.0.0
	 * @access private
	 */
	public function init_components() {

		$this->components['hooks'] = new Hooks();
		$this->components['shortcodes'] = new Shortcodes();

	}


	/**
	 * Register autoloader.
	 *
	 * Buddycore autoloader loads all the classes needed to run the plugin.
	 *
	 * @since 1.6.0
	 * @access private
	 */
	private function register_autoloader() {
		require BA_CORE_PATH . 'inc/Autoloader.php';

		Autoloader::run();
	}

}

Plugin::instance();
